import React from 'react';
import ApolloClient from 'apollo-boost';
import gql from 'graphql-tag';

export function withPreview(WrappedComponent, { query, rootName, uri, showLoading = false }) {
  return class extends React.Component {
    state = {
      data: this.props.data
    }

    loadingEl = (() => {
      const el = document.createElement('div');
      el.style.backgroundImage = 'url(https://loading.io/spinners/balls/index.circle-slack-loading-icon.svg)';
      el.style.width = '80px';
      el.style.height = '80px';
      el.style.position = 'fixed';
      el.style.top = '5px';
      el.style.left = '5px';
      el.hidden = true;
      return el;
    })();

    updateLoading = () => {
      const { data } = this.state;
      if (data[rootName] && data[rootName].__preview) {
        if (!data[rootName].__preview.loading) {
          setTimeout(() => {
            this.loadingEl.hidden = !data[rootName].__preview.loading;
          }, 500);
        } else {
          this.loadingEl.hidden = false;
        }
      }
    }

    componentDidMount() {
      document.body.appendChild(this.loadingEl);

      if (Array.isArray(query) && query.length === 2) {
        const myQuery = gql(query[1]);
        if (myQuery.definitions.length > 0) {
          const newRoot = myQuery.definitions[0].selectionSet.selections.find(selection => selection.name && selection.name.kind === 'Name' && selection.name.value === rootName);
          if (newRoot) {
            myQuery.definitions[0].selectionSet.selections = newRoot.selectionSet.selections;

            const rootValue = (this.state.data && this.state.data[rootName]) || {};

            this.setState({
              data: {
                ...this.props.data,
                [rootName]: {
                  ...rootValue,
                  __preview: {
                    loaded: false,
                    ...(rootValue.__preview || {}),
                    loading: true,
                    error: null,
                  },
                },
              },
            }, this.updateLoading);

            const client = new ApolloClient({ uri });

            client.query({
              query: myQuery,
              fetchPolicy: 'network-only',
            })
            .then(res => this.setState({
              data: {
                ...this.props.data,
                [rootName]: {
                  ...rootValue,
                  ...res.data,
                  __preview: {
                    loading: false,
                    error: null,
                    loaded: true,
                  },
                },
              },
            }, this.updateLoading))
            .catch(err => this.setState({
              data: {
                ...this.props.data,
                [rootName]: {
                  __preview: {
                    loading: false,
                    error: err,
                    loaded: false,
                  },
                },
              },
            }, this.updateLoading));
          }
        }
      }
    }

    componentWillUnmount() {
      document.body.removeChild(this.loadingEl);
    }

    render() {
      return <WrappedComponent {...this.props} data={this.state.data} />;
    }
  }
}